<?php
/**
 * @package   Essentials YOOtheme Pro 1.9.4 build 0227.1227
 * @author    ZOOlanders https://www.zoolanders.com
 * @copyright Copyright (C) Joolanders, SL
 * @license   http://www.gnu.org/licenses/gpl.html GNU/GPL
 */

namespace ZOOlanders\YOOessentials\Form\Source;

use function YOOtheme\app;
use YOOtheme\Builder;
use YOOtheme\Str;

class FormType
{
    public const TYPE_NAME = 'YooessentialsForm';

    public static function config(array $controls): array
    {
        /** @var Builder $builder */
        $builder = app(Builder::class);

        $fields = [];
        foreach ($controls as $control) {
            $type = $builder->types[$control['type']] ?? null;

            if (!$type || !$type->submittable) {
                continue;
            }

            $isListOf = $type->container;
            $name = str_replace('yooessentials_form_', '', $type->name);

            if (($name === 'upload' || $name === 'select') && ($control['props']['control_multiple'] ?? false)) {
                $isListOf = true;
            }

            $fields[Str::camelCase($control['name'])] = [
                'type' => $isListOf ? ['listOf' => 'String'] : 'String',
                'metadata' => [
                    'label' => $control['props']['control_label'] ?? Str::titleCase($control['name']),
                    'filters' => [
                        Str::contains($type->name, 'date') ? 'date' : null,
                        Str::contains($type->name, 'text') ? 'limit' : null
                    ],
                ],
            ];
        }

        return [
            'fields' => $fields,
            'metadata' => [
                'type' => true,
                'label' => 'Form',
            ],
        ];
    }
}
